==> caiviar-0.3.3.tar.gz: caiviar-0.3.3/server/MSSapi.cpp <==
// MSSapi.cpp: implementation of the MSSapi class.
//
//////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <atlbase.h>
#include <sphelper.h>
#include "MSSapi.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//

int MSSapi::initialize(Logger*log)
{
    return 1;
}

void MSSapi::finalize(Logger*log)
{
}


/**
 * the default constructor will create the basic tts instances and also store a pointer
 * to the logging subsystem.
 * @param log a pointer to the logging object
 */
MSSapi::MSSapi(Logger *log)
    :TextToSpeech(log)
{ 
	log->log("<debug> Initializing TTS");
	CoInitialize(NULL);
	CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&pVoice);
	speakCompleteEvent = pVoice->SpeakCompleteEvent();
	ISpObjectToken*token;
	pVoice->GetVoice(&token);
	pVoice->SetOutput(0,true);
}

/**
 * free used ressources
 */
MSSapi::~MSSapi()
{
	log->log("<debug> Deleting TTS");
	CoUninitialize();
}

/**
 * set implementation specific parameters
 */
void MSSapi::setParams (const char *key, const char* value)
{
}

/**
 * this method will convert a given text to speech and output this over the microsoft
 * audio mapper, thus creating direct audio output on the soundcard.
 * @param text the text to be converted to speech
 */
void MSSapi::text2Audio(const char *text)
{
	USES_CONVERSION;

	log->logf("<debug> TTS: Speaking \"%s\"...",text);
	pVoice->Speak(T2W(text), SPF_DEFAULT, NULL);
	WaitForSingleObject(speakCompleteEvent, INFINITE);
}

/**
 * create speech directly to a wave file
 * @param text the text to be spoken
 * @param filenam name and path of the file to be created
 */
void MSSapi::text2File(const char *text, const char *filename)
{
	USES_CONVERSION;
	HRESULT hr = S_OK;
	CComPtr<ISpStream>  cpWavStream;
	CComPtr<ISpStreamFormat>    cpOldStream;
	WCHAR               m_szWFileName[128];
	TCHAR szFileName[256];
	CSpStreamFormat originalFmt;

	strcpy(szFileName, filename);
	wcscpy( m_szWFileName, T2W(szFileName) );
	hr = pVoice->GetOutputStream( &cpOldStream );
	if (hr == S_OK) {
		hr = originalFmt.AssignFormat(cpOldStream);
	} else {
		hr = E_FAIL;
	}

	if (SUCCEEDED(hr)) {
		hr = SPBindToFile( m_szWFileName, SPFM_CREATE_ALWAYS, &cpWavStream, 
			&originalFmt.FormatId(), originalFmt.WaveFormatExPtr() ); 
	}
	if( SUCCEEDED( hr ) ) {
		hr = pVoice->SetOutput(cpWavStream, TRUE);
	}

	log->logf("<debug> TTS: Speaking \"%s\" into file %s\n",text, filename);
	pVoice->Speak(T2W(text), SPF_DEFAULT, NULL);
	WaitForSingleObject(speakCompleteEvent, INFINITE);
	pVoice->SetOutput(cpOldStream, TRUE);
}


/**
 * this method will create speech that is stored in a memory buffer. the created speech has the format pcm mono with 
 * 8kHz sampling rate and 16 bit. The memory structure can be directly used for input to the capi
 * subsystem.
 * @param text this text will be spoken
 * @return oubuffer pointer to the output buffer (the memory will be allocated)
 * @return size the size of the allocated memory
 */
void MSSapi::text2Stream(const char *text, unsigned char **outbuffer, unsigned long *size)
{
	USES_CONVERSION;
	HRESULT hr = S_OK;
	CComPtr<ISpStream> cpStream;
	CComPtr<IStream> cpBaseStream;
	GUID guidFormat; 
	WAVEFORMATEX* pWavFormatEx;
	STATSTG stats;
	
	// first create the necessary output stream
	hr = cpStream.CoCreateInstance(CLSID_SpStream);
	if(SUCCEEDED(hr)) {
		hr = CreateStreamOnHGlobal(NULL, true, &cpBaseStream);
	} else {
		log->logf("<error> TTS: could not init stream for ALaw output: %s", FAILED(hr));
		return;
	}
    if(SUCCEEDED(hr)) {
		//hr = SpConvertStreamFormatEnum(SPSF_CCITT_ALaw_8kHzMono, &guidFormat, &pWavFormatEx);
		hr = SpConvertStreamFormatEnum(SPSF_8kHz16BitMono, &guidFormat, &pWavFormatEx);
	} else {
		log->logf("<error> TTS: could not create basic stream object: %s", FAILED(hr));
		return;
	}
	// now connect the stream to the tts output
	if(SUCCEEDED(hr)) {
		hr = cpStream->SetBaseStream(cpBaseStream, guidFormat,pWavFormatEx);
		//cpBaseStream.Release();
	} else {
		log->logf("<error> TTS: error setting stream format to ALaw encoding: %s", FAILED(hr));
		return;
	}
	if(SUCCEEDED(hr)) {
		hr = pVoice->SetOutput(cpStream, TRUE);
	} else {
		log->logf("<error> TTS: error connecting base stream: %s", FAILED(hr));
		return;
	}
	if (!SUCCEEDED(hr)) {
		log->logf("<error> TTS: can't redirect TTS output to ALaw stream: %s", FAILED(hr));
		return;
	}

	// speak the text
	log->logf("<debug> TTS: generating speech for text '%s'.", text);
	pVoice->Speak(T2W(text), SPF_DEFAULT, NULL);
	WaitForSingleObject(speakCompleteEvent, INFINITE);

	// allocate memory for final output buffer
	cpStream->Stat(&stats, STATFLAG_DEFAULT);
	log->logf("<debug> TTS: speech complete, generated %d bytes of data.", stats.cbSize.QuadPart);
	*outbuffer = (unsigned char*)malloc((unsigned int)stats.cbSize.QuadPart + 44);
	memcpy(*outbuffer, wavHeader, 44);

	// now copy output to buffer
	LARGE_INTEGER pos;
	pos.QuadPart = 0;
	cpStream->Seek(pos, SEEK_SET, NULL);
	cpStream->Read(*outbuffer + 44, (int)stats.cbSize.QuadPart, size);
	log->logf("<debug> TTS: copied %d bytes of speech data to output buffer %x.", *size, *outbuffer);
	unsigned int wavelen = *size + 36;
	memcpy(*outbuffer + 4, &wavelen, 4);
	wavelen = *size;
	memcpy(*outbuffer + 40, &wavelen, 4);


	FILE *fp = fopen("d:\\temp.wav", "wb");
	fwrite(*outbuffer, sizeof(unsigned char), *size + 44, fp);
	fclose(fp);
	
	*size += 44;
	// clean up used objects
	cpStream.Release();
	cpBaseStream.Release();
}

==> caiviar-0.3.4.tar.gz: caiviar-0.3.4/server/MSSapi.cpp <==
// MSSapi.cpp: implementation of the MSSapi class.
//
//////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <atlbase.h>
#include <sphelper.h>
#include "MSSapi.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//

int MSSapi::initialize(Logger*log)
{
    return 1;
}

void MSSapi::finalize(Logger*log)
{
}


/**
 * the default constructor will create the basic tts instances and also store a pointer
 * to the logging subsystem.
 * @param log a pointer to the logging object
 */
MSSapi::MSSapi(Logger *log)
    :TextToSpeech(log)
{ 
	log->log("<debug> Initializing TTS");
	CoInitialize(NULL);
	CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&pVoice);
	speakCompleteEvent = pVoice->SpeakCompleteEvent();
	ISpObjectToken*token;
	pVoice->GetVoice(&token);
	pVoice->SetOutput(0,true);
}

/**
 * free used ressources
 */
MSSapi::~MSSapi()
{
	log->log("<debug> Deleting TTS");
	CoUninitialize();
}

/**
 * set implementation specific parameters
 */
int MSSapi::setParams (const char *key, const char* value)
{
	return 0;
}

/**
 * this method will convert a given text to speech and output this over the microsoft
 * audio mapper, thus creating direct audio output on the soundcard.
 * @param text the text to be converted to speech
 */
void MSSapi::text2Audio(const char *text)
{
	USES_CONVERSION;

	log->logf("<debug> TTS: Speaking \"%s\"...",text);
	pVoice->Speak(T2W(text), SPF_DEFAULT, NULL);
	WaitForSingleObject(speakCompleteEvent, INFINITE);
}

/**
 * create speech directly to a wave file
 * @param text the text to be spoken
 * @param filenam name and path of the file to be created
 */
void MSSapi::text2File(const char *text, const char *filename)
{
	USES_CONVERSION;
	HRESULT hr = S_OK;
	CComPtr<ISpStream>  cpWavStream;
	CComPtr<ISpStreamFormat>    cpOldStream;
	WCHAR               m_szWFileName[128];
	TCHAR szFileName[256];
	CSpStreamFormat originalFmt;

	strcpy(szFileName, filename);
	wcscpy( m_szWFileName, T2W(szFileName) );
	hr = pVoice->GetOutputStream( &cpOldStream );
	if (hr == S_OK) {
		hr = originalFmt.AssignFormat(cpOldStream);
	} else {
		hr = E_FAIL;
	}

	if (SUCCEEDED(hr)) {
		hr = SPBindToFile( m_szWFileName, SPFM_CREATE_ALWAYS, &cpWavStream, 
			&originalFmt.FormatId(), originalFmt.WaveFormatExPtr() ); 
	}
	if( SUCCEEDED( hr ) ) {
		hr = pVoice->SetOutput(cpWavStream, TRUE);
	}

	log->logf("<debug> TTS: Speaking \"%s\" into file %s\n",text, filename);
	pVoice->Speak(T2W(text), SPF_DEFAULT, NULL);
	WaitForSingleObject(speakCompleteEvent, INFINITE);
	pVoice->SetOutput(cpOldStream, TRUE);
}


/**
 * this method will create speech that is stored in a memory buffer. the created speech has the format pcm mono with 
 * 8kHz sampling rate and 16 bit. The memory structure can be directly used for input to the capi
 * subsystem.
 * @param text this text will be spoken
 * @return oubuffer pointer to the output buffer (the memory will be allocated)
 * @return size the size of the allocated memory
 */
void MSSapi::text2Stream(const char *text, unsigned char **outbuffer, unsigned long *size)
{
	USES_CONVERSION;
	HRESULT hr = S_OK;
	CComPtr<ISpStream> cpStream;
	CComPtr<IStream> cpBaseStream;
	GUID guidFormat; 
	WAVEFORMATEX* pWavFormatEx;
	STATSTG stats;
	
	// first create the necessary output stream
	hr = cpStream.CoCreateInstance(CLSID_SpStream);
	if(SUCCEEDED(hr)) {
		hr = CreateStreamOnHGlobal(NULL, true, &cpBaseStream);
	} else {
		log->logf("<error> TTS: could not init stream for ALaw output: %s", FAILED(hr));
		return;
	}
    if(SUCCEEDED(hr)) {
		//hr = SpConvertStreamFormatEnum(SPSF_CCITT_ALaw_8kHzMono, &guidFormat, &pWavFormatEx);
		hr = SpConvertStreamFormatEnum(SPSF_8kHz16BitMono, &guidFormat, &pWavFormatEx);
	} else {
		log->logf("<error> TTS: could not create basic stream object: %s", FAILED(hr));
		return;
	}
	// now connect the stream to the tts output
	if(SUCCEEDED(hr)) {
		hr = cpStream->SetBaseStream(cpBaseStream, guidFormat,pWavFormatEx);
		//cpBaseStream.Release();
	} else {
		log->logf("<error> TTS: error setting stream format to ALaw encoding: %s", FAILED(hr));
		return;
	}
	if(SUCCEEDED(hr)) {
		hr = pVoice->SetOutput(cpStream, TRUE);
	} else {
		log->logf("<error> TTS: error connecting base stream: %s", FAILED(hr));
		return;
	}
	if (!SUCCEEDED(hr)) {
		log->logf("<error> TTS: can't redirect TTS output to ALaw stream: %s", FAILED(hr));
		return;
	}

	// speak the text
	log->logf("<debug> TTS: generating speech for text '%s'.", text);
	pVoice->Speak(T2W(text), SPF_DEFAULT, NULL);
	WaitForSingleObject(speakCompleteEvent, INFINITE);

	// allocate memory for final output buffer
	cpStream->Stat(&stats, STATFLAG_DEFAULT);
	log->logf("<debug> TTS: speech complete, generated %d bytes of data.", stats.cbSize.QuadPart);
	*outbuffer = (unsigned char*)malloc((unsigned int)stats.cbSize.QuadPart + 44);
	memcpy(*outbuffer, wavHeader, 44);

	// now copy output to buffer
	LARGE_INTEGER pos;
	pos.QuadPart = 0;
	cpStream->Seek(pos, SEEK_SET, NULL);
	cpStream->Read(*outbuffer + 44, (int)stats.cbSize.QuadPart, size);
	log->logf("<debug> TTS: copied %d bytes of speech data to output buffer %x.", *size, *outbuffer);
	unsigned int wavelen = *size + 36;
	memcpy(*outbuffer + 4, &wavelen, 4);
	wavelen = *size;
	memcpy(*outbuffer + 40, &wavelen, 4);


	FILE *fp = fopen("d:\\temp.wav", "wb");
	fwrite(*outbuffer, sizeof(unsigned char), *size + 44, fp);
	fclose(fp);
	
	*size += 44;
	// clean up used objects
	cpStream.Release();
	cpBaseStream.Release();
}
